@@ -2,7 +2,6 @@ from applitools.common import (
     DeviceName,
     MatchLevel,
     Region,
-    ScreenOrientation,
     StdoutLogger,
     FileLogger,
     logger,
@@ -11,6 +10,14 @@ from applitools.common import (
     TestResultContainer,
     TestResultsSummary,
 )
+from applitools.common.ultrafastgrid import (
+    IosDeviceInfo,
+    ScreenOrientation,
+    IosDeviceName,
+    ChromeEmulationInfo,
+    DesktopBrowserInfo,
+)  # noqa
+from applitools.common.geometry import AccessibilityRegion
 from applitools.common.config import BatchInfo  # noqa
 from applitools.common.selenium import BrowserType, Configuration, StitchMode  # noqa
 from applitools.common.server import FailureReports  # noqa
@@ -19,7 +26,14 @@ from applitools.core.cut import (  # noqa
     NullCutProvider,
     UnscaledFixedCutProvider,
 )
-from applitools.core.batch_close import BatchClose
+from applitools.common.accessibility import (  # noqa
+    AccessibilityRegionType,
+    AccessibilitySettings,
+    AccessibilityLevel,
+    AccessibilityGuidelinesVersion,
+)
+from applitools.core.fluent.region import AccessibilityRegionByRectangle  # noqa
+from applitools.core.batch_close import BatchClose  # noqa
 
 from .classic_runner import ClassicRunner  # noqa
 from .eyes import Eyes  # noqa
@@ -54,4 +68,14 @@ __all__ = (
     "TestResultContainer",
     "TestResultsSummary",
     "BatchClose",
+    "AccessibilityRegionType",
+    "AccessibilityLevel",
+    "AccessibilitySettings",
+    "AccessibilityGuidelinesVersion",
+    "AccessibilityRegionByRectangle",
+    "AccessibilityRegion",
+    "IosDeviceName",
+    "IosDeviceInfo",
+    "ChromeEmulationInfo",
+    "DesktopBrowserInfo",
 )
