@@ -7,9 +7,11 @@ from applitools.common.selenium import Configuration
 from applitools.common.utils import argument_guard
 from applitools.common.utils.general_utils import all_fields, proxy_to
 from applitools.core.eyes_mixins import EyesConfigurationMixin
+from applitools.core.locators import VisualLocatorSettings, LOCATORS_TYPE
 from applitools.selenium import ClassicRunner, eyes_selenium_utils
 
 from .fluent import Target
+from .locators import SeleniumVisualLocatorsProvider
 from .selenium_eyes import SeleniumEyes
 from .visual_grid import VisualGridEyes, VisualGridRunner
 from .webdriver import EyesWebDriver
@@ -17,7 +19,7 @@ from .webdriver import EyesWebDriver
 if typing.TYPE_CHECKING:
     from typing import Text, Optional, Union, List, Tuple
     from selenium.webdriver.remote.webelement import WebElement
-    from applitools.common import TestResults, Region
+    from applitools.common import TestResults, Region, MatchLevel
     from applitools.common.utils.custom_types import (
         AnyWebDriver,
         ViewPort,
@@ -35,41 +37,43 @@ if typing.TYPE_CHECKING:
     from .webelement import EyesWebElement
 
 
-@proxy_to(
-    "configure",
-    all_fields(Configuration)
-    + ["use_dom", "send_dom", "match_level", "ignore_displacements", "enable_patterns"],
-)
+@proxy_to("configure", all_fields(Configuration))
 class Eyes(EyesConfigurationMixin):
     _is_visual_grid_eyes = False  # type: bool
     _visual_grid_eyes = None  # type: VisualGridEyes
     _selenium_eyes = None  # type: SeleniumEyes
+    _visual_locators_provider = None  # type: SeleniumVisualLocatorsProvider
     _runner = None  # type: Optional[VisualGridRunner]
     _driver = None  # type: Optional[EyesWebDriver]
     _is_opened = False  # type: bool
     _config_cls = Configuration
 
     def __init__(self, runner=None):
-        # type: (Optional[VisualGridRunner, ClassicRunner]) -> None
+        # type: (Union[Text, VisualGridRunner, ClassicRunner, None]) -> None
         super(Eyes, self).__init__()
 
         # backward compatibility with settings server_url
-        if isinstance(runner, str):
-            self.configure.server_url = runner
-            runner = None
-
         if runner is None:
             self._selenium_eyes = SeleniumEyes(self, None)
+        elif isinstance(runner, str):
+            self.configure.server_url = runner
+            self._selenium_eyes = SeleniumEyes(self, None)
         elif isinstance(runner, VisualGridRunner):
             self._runner = runner
             self._visual_grid_eyes = VisualGridEyes(self, runner)
             self._is_visual_grid_eyes = True
+            # for visual locators
+            self._selenium_eyes = SeleniumEyes(self, None)
         elif isinstance(runner, ClassicRunner):
             self._runner = runner
             self._selenium_eyes = SeleniumEyes(self, runner)
             self._is_visual_grid_eyes = False
         else:
-            raise ValueError("Wrong runner")
+            raise TypeError(
+                "{} is unsupported runner. Must be {} or {}".format(
+                    type(runner), ClassicRunner.__name__, VisualGridRunner.__name__
+                )
+            )
 
     def get_configuration(self):
         # type:() -> Configuration
@@ -121,6 +125,16 @@ class Eyes(EyesConfigurationMixin):
         """
         return self._current_eyes.full_agent_id
 
+    @property
+    def match_level(self):
+        # type: () -> MatchLevel
+        return self.configure.match_level
+
+    @match_level.setter
+    def match_level(self, match_level):
+        # type: (MatchLevel) -> None
+        self.configure.match_level = match_level
+
     @property
     def should_stitch_content(self):
         # type: () -> bool
@@ -200,7 +214,7 @@ class Eyes(EyesConfigurationMixin):
 
     @property
     def cut_provider(self):
-        # type:()->Optional[Union[FixedCutProvider,UnscaledFixedCutProvider,NullCutProvider]]
+        # type:()->Union[FixedCutProvider,UnscaledFixedCutProvider,NullCutProvider,None]
         """
         Gets current cut provider
         """
@@ -222,6 +236,7 @@ class Eyes(EyesConfigurationMixin):
 
     @property
     def is_cut_provider_explicitly_set(self):
+        # type: () -> bool
         """
         Gets is cut provider explicitly set.
         """
@@ -231,6 +246,7 @@ class Eyes(EyesConfigurationMixin):
 
     @property
     def agent_setup(self):
+        # type: () -> Optional[Text]
         """
         Gets agent setup.
         """
@@ -248,14 +264,14 @@ class Eyes(EyesConfigurationMixin):
     @staticmethod
     def get_viewport_size(driver):
         # type: (AnyWebDriver) -> ViewPort
+        argument_guard.not_none(driver)
         return eyes_selenium_utils.get_viewport_size_or_display_size(driver)
 
     @staticmethod
     def set_viewport_size(driver, size):
         # type: (AnyWebDriver, ViewPort) -> None
-        assert driver is not None
-        if size is None:
-            raise ValueError("set_viewport_size require `size` parameter")
+        argument_guard.not_none(driver)
+        argument_guard.not_none(size)
         eyes_selenium_utils.set_viewport_size(driver, size)
 
     def add_property(self, name, value):
@@ -265,13 +281,13 @@ class Eyes(EyesConfigurationMixin):
         :param name: (string) The property name.
         :param value: (string) The property value
         """
-        self._current_eyes.add_property(name, value)
+        self._current_eyes.configure.add_property(name, value)
 
     def clear_properties(self):
         """
         Clears the list of custom properties.
         """
-        self._current_eyes.clear_properties()
+        self._current_eyes.configure.clear_properties()
 
     def add_mouse_trigger_by_element(self, action, element):
         # type: (Text, AnyWebElement) -> None
@@ -329,6 +345,7 @@ class Eyes(EyesConfigurationMixin):
         :return: The match results.
         """
         if self.configure.is_disabled:
+            logger.info("check(): ignored (disabled)")
             return MatchResult()
         if not self.is_open:
             self.abort()
@@ -447,17 +464,33 @@ class Eyes(EyesConfigurationMixin):
         if self.configure.is_disabled:
             logger.info("open(): ignored (disabled)")
             return
+        logger.info(
+            "open(app_name={}, test_name={}, viewport_size={})".format(
+                app_name, test_name, viewport_size
+            )
+        )
         if app_name:
             self.configure.app_name = app_name
         if test_name:
             self.configure.test_name = test_name
         if viewport_size:
             self.configure.viewport_size = viewport_size  # type: ignore
+
         self._init_driver(driver)
+        self._init_locator_provider()
+        if self._is_visual_grid_eyes:
+            self._selenium_eyes.open(self._driver, skip_start_session=True)
+
         result = self._current_eyes.open(self.driver)
         self._is_opened = True
         return result
 
+    def locate(self, visual_locator_settings):
+        # type: (VisualLocatorSettings) -> LOCATORS_TYPE
+        argument_guard.is_a(visual_locator_settings, VisualLocatorSettings)
+        logger.info("locate({})".format(visual_locator_settings))
+        return self._visual_locators_provider.get_locators(visual_locator_settings)
+
     def close(self, raise_ex=True):
         # type: (bool) -> Optional[TestResults]
         """
@@ -469,6 +502,7 @@ class Eyes(EyesConfigurationMixin):
         if self.configure.is_disabled:
             logger.info("close(): ignored (disabled)")
             return
+        logger.info("close()")
         result = self._current_eyes.close(raise_ex)
         self._is_opened = False
         return result
@@ -477,6 +511,7 @@ class Eyes(EyesConfigurationMixin):
         if self.configure.is_disabled:
             logger.info("close_async(): ignored (disabled)")
             return
+        logger.info("close_async()")
         if self._is_visual_grid_eyes:
             self._visual_grid_eyes.close_async()
         else:
@@ -489,12 +524,15 @@ class Eyes(EyesConfigurationMixin):
         if self.configure.is_disabled:
             logger.info("abort(): ignored (disabled)")
             return
+        logger.info("abort()")
         self._current_eyes.abort()
 
     def abort_async(self):
         if self.configure.is_disabled:
             logger.info("abort_async(): ignored (disabled)")
             return
+
+        logger.info("abort_async()")
         if self._is_visual_grid_eyes:
             return self._visual_grid_eyes.abort_async()
         else:
@@ -513,6 +551,11 @@ class Eyes(EyesConfigurationMixin):
         else:
             self._driver = EyesWebDriver(driver, self)
 
+    def _init_locator_provider(self):
+        self._visual_locators_provider = SeleniumVisualLocatorsProvider(
+            self._driver, self._selenium_eyes
+        )
+
     @property
     def _current_eyes(self):
         # type: () -> Union[SeleniumEyes, VisualGridEyes]
