@@ -1,6 +1,9 @@
 from __future__ import absolute_import
 
+import json
+import threading
 import typing as tp
+from collections import OrderedDict
 from contextlib import contextmanager
 
 from appium.webdriver import Remote as AppiumWebDriver
@@ -8,7 +11,7 @@ from selenium.common.exceptions import WebDriverException
 from selenium.webdriver.remote.webdriver import WebDriver
 from selenium.webdriver.remote.webelement import WebElement
 
-from applitools.common import Point, RectangleSize, logger
+from applitools.common import Point, RectangleSize, logger, EyesError
 from applitools.common.utils import datetime_utils
 
 if tp.TYPE_CHECKING:
@@ -215,26 +218,31 @@ def set_browser_size(driver, required_size):
     retries_left = _RETRIES
 
     # set browser size for mobile devices isn't working
-    if is_mobile_web(driver):
+    if is_mobile_platform(driver):
         return True
 
     while True:
-        logger.info("Trying to set browser size to: " + str(required_size))
+        logger.debug("Trying to set browser size to: " + str(required_size))
         set_window_size(driver, required_size)
         datetime_utils.sleep(_SLEEP_MS)
         current_size = get_window_size(driver)
-        logger.info("Current browser size: " + str(required_size))
+        logger.debug("Current browser size: " + str(current_size))
 
         if retries_left or current_size != required_size:
             break
         retries_left -= 1
 
-    return current_size == required_size
+    if current_size == required_size:
+        logger.debug(
+            "Browser size has been successfully set to {}".format(current_size)
+        )
+        return True
+    logger.debug("Browser size wasn't set: {}".format(current_size))
+    return False
 
 
 def set_browser_size_by_viewport_size(driver, actual_viewport_size, required_size):
     # type: (AnyWebDriver, ViewPort, ViewPort) -> bool
-
     browser_size = get_window_size(driver)
     logger.debug("Current browser size: {}".format(browser_size))
     required_browser_size = dict(
@@ -243,18 +251,25 @@ def set_browser_size_by_viewport_size(driver, actual_viewport_size, required_siz
         height=browser_size["height"]
         + (required_size["height"] - actual_viewport_size["height"]),
     )
-    return set_browser_size(driver, required_browser_size)
+    set_browser_size(driver, required_browser_size)
+    # Browser size and Viewport size are different things.
+    # We need to compare the by Viewport sizes to be sure that it fit.
+    actual_viewport_size = get_viewport_size(driver)
+    logger.info("Current viewport size: {}".format(actual_viewport_size))
+    return actual_viewport_size == required_size
 
 
-def set_viewport_size(driver, required_size):
+def set_viewport_size(driver, required_size):  # noqa
     # type: (AnyWebDriver, ViewPort) -> None
-
-    logger.debug("set_viewport_size({})".format(str(required_size)))
-
     actual_viewport_size = get_viewport_size(driver)
     if actual_viewport_size == required_size:
-        logger.info("Required size already set.")
+        logger.info("Required viewport size already set")
         return None
+    logger.info(
+        "Actual Viewport Size: {}\n\tTrying to set viewport size to: {}".format(
+            str(actual_viewport_size), str(required_size)
+        )
+    )
 
     try:
         # We move the window to (0,0) to have the best chance to be able to
@@ -262,9 +277,7 @@ def set_viewport_size(driver, required_size):
         driver.set_window_position(0, 0)
     except WebDriverException:
         logger.warning("Failed to move the browser window to (0,0)")
-    set_browser_size_by_viewport_size(driver, actual_viewport_size, required_size)
-    actual_viewport_size = get_viewport_size(driver)
-    if actual_viewport_size == required_size:
+    if set_browser_size_by_viewport_size(driver, actual_viewport_size, required_size):
         return None
 
     # Additional attempt. This Solves the "maximized browser" bug
@@ -272,10 +285,7 @@ def set_viewport_size(driver, required_size):
     # non-maximized, so the original browser size calculation is
     # wrong).
     logger.info("Trying workaround for maximization...")
-    set_browser_size_by_viewport_size(driver, actual_viewport_size, required_size)
-    actual_viewport_size = get_viewport_size(driver)
-    logger.debug("Current viewport size: {}".format(actual_viewport_size))
-    if actual_viewport_size == required_size:
+    if set_browser_size_by_viewport_size(driver, actual_viewport_size, required_size):
         return None
 
     width_diff = abs(actual_viewport_size["width"] - required_size["width"])
@@ -326,7 +336,18 @@ def set_viewport_size(driver, required_size):
                 return None
         else:
             logger.info("Zoom workaround failed.")
-    raise WebDriverException("Failed to set the viewport size.")
+
+    # Attempt to fix by minimizing window
+    logger.info("Trying workaround for minimization...")
+    try:
+        # some webdriver's don't support minimize_window
+        driver.minimize_window()
+    except WebDriverException as e:
+        logger.exception(e)
+    if set_browser_size_by_viewport_size(driver, actual_viewport_size, required_size):
+        return None
+    logger.error("Minimization workaround failed.")
+    raise EyesError("Failed to set the viewport size.")
 
 
 def hide_scrollbars(driver, root_element):
@@ -546,3 +567,39 @@ def get_default_content_scroll_position(current_frames, driver):
                 driver, curr_frame_scroll_root_element(driver)
             )
     return scroll_position
+
+
+def get_dom_script_result(driver, dom_extraction_timeout, timer_name, script_for_run):
+    # type: (AnyWebDriver, int, Text, Text) -> Dict
+    is_check_timer_timeout = []
+    script_response = {}
+    status = None
+
+    def start_timer():
+        def set_timer():
+            is_check_timer_timeout.append(True)
+
+        timer = threading.Timer(dom_extraction_timeout, set_timer)
+        timer.daemon = True
+        timer.setName(timer_name)
+        timer.start()
+        return timer
+
+    timer = start_timer()
+    while True:
+        if status == "SUCCESS" or is_check_timer_timeout:
+            del is_check_timer_timeout[:]
+            break
+        script_result_string = driver.execute_script(script_for_run)
+        try:
+            script_response = json.loads(
+                script_result_string, object_pairs_hook=OrderedDict
+            )
+            status = script_response.get("status")
+        except Exception as e:
+            logger.exception(e)
+    timer.cancel()
+    script_result = script_response.get("value")
+    if script_result is None or status == "ERROR":
+        raise EyesError("Failed to capture script_result")
+    return script_result
