@@ -162,19 +162,20 @@ class SeleniumEyes(EyesBase):
         """
         return self._driver
 
-    def open(self, driver):
-        # type: (AnyWebDriver) -> EyesWebDriver
+    def open(self, driver, skip_start_session=False):
+        # type: (AnyWebDriver, bool) -> EyesWebDriver
         self._driver = driver
         self._screenshot_factory = EyesWebDriverScreenshotFactory(self.driver)
         self._ensure_viewport_size()
+        self._user_agent = self._driver.user_agent
 
-        self._open_base()
-
-        ua_string = self._session_start_info.environment.inferred
-        if ua_string:
-            ua_string = ua_string.replace("useragent:", "")
-            self._user_agent = useragent.parse_user_agent_string(ua_string)
-
+        if skip_start_session:
+            self._server_connector.update_config(
+                self.get_configuration(), self.full_agent_id
+            )
+            self._init_providers()
+        else:
+            self._open_base()
         self._image_provider = get_image_provider(self._user_agent, self)
         self._region_position_compensation = get_region_position_compensation(
             self._user_agent, self
@@ -219,7 +220,7 @@ class SeleniumEyes(EyesBase):
 
     def _create_position_provider(self, scroll_root_element):
         stitch_mode = self.configure.stitch_mode
-        logger.info(
+        logger.debug(
             "initializing position provider. stitch_mode: {}".format(stitch_mode)
         )
         if stitch_mode == StitchMode.Scroll:
@@ -259,7 +260,7 @@ class SeleniumEyes(EyesBase):
             # hide scrollbar for main window
             self._try_hide_scrollbars()
 
-            logger.info("URL: {}".format(self._driver.current_url))
+            logger.info("Current URL: {}".format(self._driver.current_url))
             with self._switch_to_frame(check_settings):
                 result = self._check_result_flow(name, check_settings)
 
@@ -276,7 +277,8 @@ class SeleniumEyes(EyesBase):
         if self._position_memento:
             self._position_provider.restore_state(self._position_memento)
 
-        self.driver.switch_to.frames(self._original_fc)
+        if not self.driver.is_mobile_app:
+            self.driver.switch_to.frames(self._original_fc)
         self._position_provider = None
         self._original_fc = None
         logger.debug("check - done!")
@@ -341,7 +343,7 @@ class SeleniumEyes(EyesBase):
             if self._check_frame_or_element:
                 fc = self._ensure_frame_visible()
                 # FIXME - Scaling should be handled in a single place instead
-                scale_factory = self._update_scaling_params()
+                scale_factory = self.update_scaling_params()
                 screenshot_image = self._image_provider.get_image()
                 scale_factory.update_scale_ratio(screenshot_image.width)
                 self.driver.switch_to.frames(fc)
@@ -683,7 +685,7 @@ class SeleniumEyes(EyesBase):
                     os = platform_name
                 logger.info("Setting OS: " + os)
             else:
-                logger.info("No mobile OS detected.")
+                logger.info("No mobile OS detected")
         app_env = AppEnvironment(
             os,
             hosting_app=self.configure.host_app,
@@ -708,15 +710,12 @@ class SeleniumEyes(EyesBase):
     @property
     def _inferred_environment(self):
         # type: () -> Optional[Text]
-        try:
-            user_agent = self._driver.execute_script("return navigator.userAgent")
-        except WebDriverException:
-            user_agent = None
+        user_agent = self._driver.user_agent
         if user_agent:
-            return "useragent:%s" % user_agent
+            return "useragent:%s" % user_agent.origin_ua_string
         return None
 
-    def _update_scaling_params(self):
+    def update_scaling_params(self):
         # type: () -> Optional[ScaleProvider]
         if (
             self.device_pixel_ratio != self._UNKNOWN_DEVICE_PIXEL_RATIO
@@ -751,7 +750,7 @@ class SeleniumEyes(EyesBase):
             logger.info("Failed to set ContextBasedScaleProvider.")
             logger.info("Using FixedScaleProvider instead...")
             self._scale_provider = FixedScaleProvider(1 / device_pixel_ratio)
-        logger.info("Done!")
+        logger.debug("Scaling provider has been set")
         return self._scale_provider
 
     def _try_capture_dom(self):
@@ -833,7 +832,7 @@ class SeleniumEyes(EyesBase):
 
         with self._try_hide_caret():
 
-            scale_provider = self._update_scaling_params()
+            scale_provider = self.update_scaling_params()
 
             if self._check_frame_or_element and not self.driver.is_mobile_app:
                 self._last_screenshot = self._entire_element_screenshot(scale_provider)
@@ -911,14 +910,14 @@ class SeleniumEyes(EyesBase):
         logger.info("Element screenshot requested")
         with self._ensure_element_visible(self._target_element):
             datetime_utils.sleep(self.configure.wait_before_screenshots)
-            image = self._get_scaled_cropped_image(scale_provider)
-            if not self._is_check_region and not self._driver.is_mobile_platform:
+            image = self.get_scaled_cropped_viewport_image(scale_provider)
+            if not (self._is_check_region or self._driver.is_mobile_platform):
                 # Some browsers return always full page screenshot (IE).
                 # So we cut such images to viewport size
                 image = cut_to_viewport_size_if_required(self.driver, image)
             return EyesWebDriverScreenshot.create_viewport(self._driver, image)
 
-    def _get_scaled_cropped_image(self, scale_provider):
+    def get_scaled_cropped_viewport_image(self, scale_provider):
         image = self._image_provider.get_image()
         self._debug_screenshot_provider.save(image, "original")
         scale_provider.update_scale_ratio(image.width)
@@ -940,8 +939,8 @@ class SeleniumEyes(EyesBase):
                 location = eyes_selenium_utils.get_current_position(
                     self.driver, self.scroll_root_element
                 )
-            except WebDriverException as e:
-                logger.warning(str(e))
+            except WebDriverException:
+                logger.warning("Failed to _get_viewport_scroll_bounds")
                 logger.info("Assuming position is 0,0")
                 location = Point(0, 0)
         viewport_bounds = Region.from_(location, self._get_viewport_size())
@@ -950,7 +949,7 @@ class SeleniumEyes(EyesBase):
     @contextlib.contextmanager
     def _ensure_element_visible(self, element):
         position_provider = None
-        if self._target_element and not self.driver.is_mobile_app:
+        if element and not self.driver.is_mobile_app:
             original_fc = self.driver.frame_chain.clone()
             eyes_element = EyesWebElement(element, self.driver)
             element_bounds = eyes_element.bounds
@@ -982,6 +981,6 @@ class SeleniumEyes(EyesBase):
                 position_provider.set_position(element_location)
 
         yield position_provider
-        if self._target_element and position_provider and not self.driver.is_mobile_app:
+        if element and position_provider and not self.driver.is_mobile_app:
             self.driver.switch_to.frames(fc)
             position_provider.restore_state(state)
