@@ -4,14 +4,13 @@ import typing
 from concurrent.futures import Future, ThreadPoolExecutor
 
 from applitools.common import VGResource, logger
-from applitools.common.utils import iteritems
 
 
 class ResourceCache(typing.Mapping[typing.Text, VGResource]):
     def __init__(self):
         self.cache_map = {}
         kwargs = {}
-        if sys.version_info >= (3, 6):
+        if sys.version_info[:2] >= (3, 6):
             kwargs["thread_name_prefix"] = "ResourceCache-Executor"
         self.executor = ThreadPoolExecutor(**kwargs)
         self.lock = threading.RLock()
@@ -19,27 +18,23 @@ class ResourceCache(typing.Mapping[typing.Text, VGResource]):
     def __str__(self):
         return str(self.cache_map)[:25]
 
-    def _process_future(self, val):
+    def _process_future(self, url, val):
         if isinstance(val, Future):
             try:
-                return val.result()
-            except Exception as e:
-                logger.exception(e)
-                return None
-        return val
-
-    def get(self, k, default=None):
-        with self.lock:
-            val = self.cache_map.get(k, default)
-            val = self._process_future(val)
-            self[k] = val
+                val = val.result()
+            except Exception:
+                logger.exception(
+                    "We got an exception during processing URL: {}".format(url)
+                )
+                val = None
+            finally:
+                self.cache_map[url] = val
         return val
 
     def __getitem__(self, item):
         with self.lock:
             val = self.cache_map[item]
-            val = self._process_future(val)
-            self[item] = val
+            self._process_future(item, val)
         return val
 
     def __setitem__(self, key, value):
@@ -66,9 +61,5 @@ class ResourceCache(typing.Mapping[typing.Text, VGResource]):
         return False
 
     def process_all(self):
-        for r_url, val in iteritems(self):
-            if val is None:
-                logger.debug("No response for {}".format(r_url))
-                val = VGResource.EMPTY(r_url)
-                self[r_url] = val
-        return self
+        with self.lock:
+            return [self[r_url] for r_url in self]
