@@ -7,21 +7,22 @@ from collections import OrderedDict
 import attr
 
 from applitools.common import (
-    DiffsFoundError,
     EyesError,
-    NewTestError,
-    TestFailedError,
     TestResults,
     logger,
-    RectangleSize,
 )
-from applitools.common.utils import argument_guard, datetime_utils
-from applitools.common.visual_grid import RenderBrowserInfo, VisualGridSelector
+from applitools.common.utils import argument_guard
+from applitools.common.ultrafastgrid import (
+    RenderBrowserInfo,
+    VisualGridSelector,
+    DesktopBrowserInfo,
+)
 from applitools.core import CheckSettings, GetRegion
 from applitools.selenium import __version__, eyes_selenium_utils, resource
 from applitools.selenium.fluent import SeleniumCheckSettings
 
 from .eyes_connector import EyesConnector
+from .helpers import collect_test_results, wait_till_tests_completed
 from .running_test import RunningTest
 from .visual_grid_runner import VisualGridRunner
 
@@ -51,7 +52,7 @@ el = parent;
 return '/' + xpath;"""
 
 
-PROCESS_RESOURCES = resource.get_resource("processPageAndPoll.js")
+PROCESS_RESOURCES = resource.get_resource("processPageAndSerializePoll.js")
 DOM_EXTRACTION_TIMEOUT = 5 * 60 * 1000
 
 
@@ -78,7 +79,7 @@ class VisualGridEyes(object):
         self._config_provider = config_provider
         self._elements = []
         argument_guard.not_none(runner)
-        self.vg_manager = runner
+        self.vg_manager = runner  # type: VisualGridRunner
         self.test_list = []  # type: List[RunningTest]
         self._test_uuid = None
 
@@ -116,15 +117,6 @@ class VisualGridEyes(object):
             return self.base_agent_id
         return "{} [{}]".format(agent_id, self.base_agent_id)
 
-    def add_property(self, name, value):
-        # type: (Text, Text) -> None
-        """
-        Associates a key/value pair with the test. This can be used later for filtering.
-        :param name: (string) The property name.
-        :param value: (string) The property value
-        """
-        self.configure.properties.append({"name": name, "value": value})
-
     def open(self, driver):
         # type: (EyesWebDriver) -> EyesWebDriver
         self._test_uuid = uuid.uuid4()
@@ -135,13 +127,14 @@ class VisualGridEyes(object):
         self._driver = driver
         self._set_viewport_size()
 
+        ua_string = self._driver.execute_script("return navigator.userAgent").replace(
+            "useragent:", ""
+        )
         for b_info in self.configure.browsers_info:
             test = RunningTest(
-                self._create_vgeyes_connector(b_info), self.configure, b_info
-            )
-            test.on_results_received(
-                lambda r: self.vg_manager.aggregate_result(test, r)
+                self._create_vgeyes_connector(b_info, ua_string), self.configure, b_info
             )
+            test.on_results_received(self.vg_manager.aggregate_result)
             test.test_uuid = self._test_uuid
             self.test_list.append(test)
         self._is_opened = True
@@ -149,40 +142,14 @@ class VisualGridEyes(object):
         logger.info("VisualGridEyes opening {} tests...".format(len(self.test_list)))
         return driver
 
-    def _start_timer(self):
-        def set_timer():
-            self.is_check_timer_timeout = True
-
-        timer = threading.Timer(DOM_EXTRACTION_TIMEOUT, set_timer)
-        timer.daemon = True
-        timer.setName("VG_StopWatch")
-        timer.start()
-        return timer
-
     def get_script_result(self):
         # type: () -> Dict
-        script_response = {}
-        status = None
-        timer = self._start_timer()
-        while True:
-            if status == "SUCCESS" or self.is_check_timer_timeout:
-                self.is_check_timer_timeout = False
-                break
-            script_result_string = self.driver.execute_script(
-                PROCESS_RESOURCES + "return __processPageAndPoll();"
-            )
-            try:
-                script_response = json.loads(
-                    script_result_string, object_pairs_hook=OrderedDict
-                )
-                status = script_response.get("status")
-            except Exception as e:
-                logger.exception(e)
-        timer.cancel()
-        script_result = script_response.get("value")
-        if script_result is None or status == "ERROR":
-            raise EyesError("Failed to capture script_result")
-        return script_result
+        return eyes_selenium_utils.get_dom_script_result(
+            self.driver,
+            DOM_EXTRACTION_TIMEOUT,
+            "VG_StopWatch",
+            PROCESS_RESOURCES + "return __processPageAndSerializePoll();",
+        )
 
     def check(self, name, check_settings):
         # type: (Text, SeleniumCheckSettings) -> None
@@ -227,46 +194,20 @@ class VisualGridEyes(object):
     def close(self, raise_ex=True):  # noqa
         # type: (Optional[bool]) -> Optional[TestResults]
         if not self.test_list:
-            return TestResults()
+            return TestResults(status="Failed")
         logger.debug("VisualGridEyes.close()\n\t test_list %s" % self.test_list)
         self.close_async()
 
-        while True:
-            states = list(set(t.state for t in self.test_list))
-            logger.debug("Current test states: \n {}".format(states))
-            if len(states) == 1 and states[0] == "completed":
-                break
-            datetime_utils.sleep(
-                1500, msg="Waiting for state completed in VisualGridEyes.close"
-            )
+        wait_till_tests_completed(self.test_list)
 
         self._is_opened = False
 
-        for test in self.test_list:
-            if test.pending_exceptions:
-                logger.error(
-                    "During test execution above exception raised. \n {:s}".join(
-                        str(e) for e in test.pending_exceptions
-                    )
-                )
-        if raise_ex:
-            for test in self.test_list:
-                if test.test_result is None:
-                    raise TestFailedError("Test haven't finished correctly")
-                results = test.test_result
-                scenario_id_or_name = results.name
-                app_id_or_name = results.app_name
-                if results.is_unresolved and not results.is_new:
-                    raise DiffsFoundError(results, scenario_id_or_name, app_id_or_name)
-                if results.is_new:
-                    raise NewTestError(results, scenario_id_or_name, app_id_or_name)
-                if results.is_failed:
-                    raise TestFailedError(results, scenario_id_or_name, app_id_or_name)
-
-        all_results = [t.test_result for t in self.test_list if t.test_result]
+        all_results = collect_test_results(
+            {t: t.test_result for t in self.test_list}, raise_ex
+        )
         if not all_results:
-            return TestResults()
-        return all_results[0]
+            return TestResults(status="Failed")
+        return all_results[0].test_results
 
     def abort_async(self):
         """
@@ -307,16 +248,13 @@ class VisualGridEyes(object):
             )
         return check_settings
 
-    def _create_vgeyes_connector(self, b_info):
-        # type: (RenderBrowserInfo) -> EyesConnector
-        vgeyes_connector = EyesConnector(b_info, self.configure.clone())
-        if b_info.emulation_info:
-            vgeyes_connector.device = b_info.emulation_info.device_name
-
+    def _create_vgeyes_connector(self, b_info, ua_string):
+        # type: (RenderBrowserInfo, Text) -> EyesConnector
+        vgeyes_connector = EyesConnector(
+            b_info, self.configure.clone(), ua_string, self.rendering_info
+        )
         if self.rendering_info is None:
             self.rendering_info = vgeyes_connector.render_info()
-        if self.rendering_info:
-            vgeyes_connector._server_connector._render_info = self.rendering_info
         return vgeyes_connector
 
     def _try_set_target_selector(self, check_settings):
@@ -370,6 +308,9 @@ class VisualGridEyes(object):
         floating_elements = self.get_elements_from_regions(
             check_settings.values.floating_regions
         )
+        accessibility_elements = self.get_elements_from_regions(
+            check_settings.values.accessibility_regions
+        )
         element = check_settings.values.target_element
         if element is None:
             target_selector = check_settings.values.target_selector
@@ -384,6 +325,7 @@ class VisualGridEyes(object):
             strict_elements,
             content_elements,
             floating_elements,
+            accessibility_elements,
             targets,
         ]
 
@@ -407,12 +349,15 @@ class VisualGridEyes(object):
 
         if viewport_size is None:
             for render_bi in self.configure.browsers_info:
-                if not render_bi.emulation_info:
-                    viewport_size = RectangleSize(render_bi.width, render_bi.height)
+                if isinstance(render_bi, DesktopBrowserInfo) or isinstance(
+                    render_bi, RenderBrowserInfo
+                ):
+                    viewport_size = render_bi.viewport_size
                     break
 
         if viewport_size is None:
-            viewport_size = self._get_viewport_size()
+            self.configure.viewport_size = self._get_viewport_size()
+            return
 
         self.configure.viewport_size = viewport_size
         try:
