@@ -14,10 +14,10 @@ from applitools.common import RectangleSize, logger
 from applitools.common.geometry import Point
 from applitools.common.utils import argument_guard, cached_property, image_utils
 from applitools.common.utils.compat import basestring
-from applitools.common.utils.general_utils import proxy_to, all_fields
+from applitools.common.utils.general_utils import proxy_to, all_attrs
 from applitools.selenium.fluent import FrameLocator
 
-from . import eyes_selenium_utils
+from . import eyes_selenium_utils, useragent
 from .frames import Frame, FrameChain
 from .positioning import ScrollPositionProvider
 from .webelement import EyesWebElement
@@ -68,11 +68,11 @@ class FrameResolver(object):
                 frame_ref = frame_locator.frame_element
             if frame_locator.frame_selector:
                 by, value = frame_locator.frame_selector
-                frame_ref = self._driver.find_elemen(by, value)
+                frame_ref = self._driver.find_element(by, value)
         return frame_ref
 
 
-@proxy_to("_switch_to", all_fields(SwitchTo))
+@proxy_to("_switch_to")
 class _EyesSwitchTo(object):
     """
     Wraps :py:class:`selenium.webdriver.remote.switch_to.SwitchTo` object, so we can
@@ -90,11 +90,9 @@ class _EyesSwitchTo(object):
         :param driver: EyesWebDriver instance.
         :param switch_to: Selenium switchTo object.
         """
+        self._proxy_to_fields = all_attrs(switch_to)
         self._switch_to = switch_to  # type: SwitchTo
         self._driver = driver  # type: EyesWebDriver
-        self._scroll_position = ScrollPositionProvider(
-            driver, eyes_selenium_utils.curr_frame_scroll_root_element(driver)
-        )
 
     @contextlib.contextmanager
     def frame_and_back(self, frame_reference):
@@ -220,8 +218,7 @@ class _EyesSwitchTo(object):
     @contextlib.contextmanager
     def frames_do_scroll(self, frame_chain):
         self.default_content()
-        root_element = eyes_selenium_utils.curr_frame_scroll_root_element(self._driver)
-        scroll_provider = ScrollPositionProvider(self._driver, root_element)
+        scroll_provider = self._get_scroll_provider()
         self._position_memento = scroll_provider.get_state()
         for frame in frame_chain:
             logger.debug("Scrolling by parent scroll position...")
@@ -231,17 +228,23 @@ class _EyesSwitchTo(object):
             self.frame(frame.reference)
             new_frame = self._driver.frame_chain.peek
             new_frame.scroll_root_element = frame.scroll_root_element
-            logger.debug("Done")
+            logger.debug("Done switching to frame")
         logger.debug("Done switching into nested frames!")
         return self._driver
 
     def reset_scroll(self):
         if self._position_memento:
-            self._scroll_position.restore_state(self._position_memento)
+            self._get_scroll_provider().restore_state(self._position_memento)
             self._position_memento = None
 
+    def _get_scroll_provider(self):
+        scroll_root_element = eyes_selenium_utils.curr_frame_scroll_root_element(
+            self._driver
+        )
+        return ScrollPositionProvider(self._driver, scroll_root_element)
 
-@proxy_to("_driver", all_fields(WebDriver))
+
+@proxy_to("_driver")
 class EyesWebDriver(object):
     """
     A wrapper for selenium web driver which creates wrapped elements,
@@ -263,6 +266,7 @@ class EyesWebDriver(object):
         :param eyes: A Eyes sdk instance.
         :param stitch_mode: How to stitch a page (default is with scrolling).
         """
+        self._proxy_to_fields = all_attrs(driver)
         self._driver = driver
         self._eyes = eyes
         # List of frames the user switched to, and the current offset,
@@ -278,6 +282,16 @@ class EyesWebDriver(object):
         # type: () -> Eyes
         return self._eyes
 
+    @property
+    def user_agent(self):
+        # type: () -> useragent.UserAgent
+        try:
+            ua_string = self._driver.execute_script("return navigator.userAgent")
+            user_agent = useragent.parse_user_agent_string(ua_string)
+        except WebDriverException:
+            user_agent = None
+        return user_agent
+
     @property
     def session_id(self):
         return self._driver.session_id
@@ -375,11 +389,11 @@ class EyesWebDriver(object):
         :return: A element denoted by "By".
         """
         # Get result from the original implementation of the underlying driver.
-        result = self._driver.find_element(by, value)
+        element = self._driver.find_element(by, value)
         # Wrap the element.
-        if result:
-            result = EyesWebElement(result, self)
-        return result
+        if element:
+            element = EyesWebElement(element, self)
+        return element
 
     def find_elements(self, by=By.ID, value=None):
         # type: (Text, Text) -> List[EyesWebElement]
@@ -391,14 +405,11 @@ class EyesWebDriver(object):
         :return: List of elements denoted by "By".
         """
         # Get result from the original implementation of the underlying driver.
-        results = self._driver.find_elements(by, value)
+        elements = self._driver.find_elements(by, value)
         # Wrap all returned elements.
-        if results:
-            updated_results = []
-            for element in results:
-                updated_results.append(EyesWebElement(element, self))
-            results = updated_results
-        return results
+        if elements:
+            elements = [EyesWebElement(el, self) for el in elements]
+        return elements
 
     def find_element_by_id(self, id_):
         # type: (Text) -> EyesWebElement
